/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief App to transmit a message from gsm module
 *
 * \author Atcom
 * 
 * \note Requires support of sending  messages from  channel driver
 *
 * \ingroup applications
 */
 
#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 187366 $")

//#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include "asterisk/file.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/app.h"
#include "asterisk/cli.h"

/*! The timeout for send sms, in seconds */

/*** DOCUMENTATION
	<application name="Sendsms" language="en_US">
		<synopsis>
			Send a Text Message from GSM module.
		</synopsis>
		<syntax>
			<parameter name="text" required="true" />
		</syntax>
		<description>
			<para>sendsms dahdi/id phone msg :Sends text to dahdii(gsm) channel (callee).</para>
			<para>Result of transmission will be stored in the <variable>SENDSMSSTATUS</variable></para>
			<variablelist>
				<variable name="SENDSMSSTATUS">
					<value name="SUCCESS">
						Transmission succeeded.
					</value>
					<value name="FAILURE">
						Transmission failed.
					</value>
				</variable>
			</variablelist>
			<note><para>At this moment, text is supposed to be 7 bit ASCII in most channels.</para></note>
		</description>
		<see-also>
			<ref type="application">Sendsms</ref>
			<ref type="application">SendImage</ref>
			<ref type="application">SendURL</ref>
		</see-also>
	</application>
	<application name="DoSms" language="en_US">
		<synopsis>
			Send a Text Message from GSM module.
		</synopsis>
		<syntax>
			<parameter name="text" required="true" />
		</syntax>
		<description>
			<para> sendsms dahdi/id phone msg :Sends text to dahdii(gsm) channel (callee).</para>
			<para>Result of transmission will be stored in the <variable>SENDSMSSTATUS</variable></para>
			<variablelist>
				<variable name="SENDSMSSTATUS">
					<value name="SUCCESS">
						Transmission succeeded.
					</value>
					<value name="FAILURE">
						Transmission failed.
					</value>
				</variable>
			</variablelist>
			<note><para>At this moment, text is supposed to be 7 bit ASCII in most channels.</para></note>
		</description>
		<see-also>
			<ref type="application">Sendsms</ref>
			<ref type="application">SendImage</ref>
			<ref type="application">SendURL</ref>
		</see-also>
	</application>
 ***/

static const char *app = "SendSms";
static const char *app1 = "DoSms";

static char *sendsms_descrip =
"  sendsms dahdi/id phone msg :Sends text to dahdii(gsm) channel (callee).\n"
"Result of transmission will be stored in the SENDSMSSTATUS\n";

static char *sms_descrip =
"  sendsms dahdi/id phone msg :Sends text to dahdii(gsm) channel (callee).\n"
"Result of transmission will be stored in the SENDSMSSTATUS\n";

static char *sendsms_synopsis = "Send a Text Message from GSM module";
static char *sms_synopsis = "Send a Text Message from GSM module";

#if 0
static char gsm_set_imei_usage[] = 
"Usage: gsm set imei cardnum value\n"
"       cardnum is 0 to 3, value is 16 length of number\n";

static char gsm_poweroff_usage[] = 
"Usage: gsm poweroff cardnum\n"
"       cardnum is 0 to 3\n";

static char gsm_poweron_usage[] = 
"Usage: gsm poweron cardnum\n"
"       cardnum is 0 to 3\n";


static int gsm_set_imei(int fd, int argc, char *argv[])
{
   int ctl;
   int res = 0;
   struct imei{
    int cardnum;
    char value[16];
   }gsm_imei;

   if(argc != 3)
     return RESULT_SHOWUSAGE;
   memset(&gsm_imei, 0, sizeof(struct imei));
   gsm_imei.cardnum = atoi(argv[3]);
   
   if(gsm_imei.cardnum < 0 || gsm_imei.cardnum >=4)
      return RESULT_SHOWUSAGE;
   ast_copy_string(&gsm_imei.value,argv[4],sizeof(gsm_imei.value));
   ast_cli(fd,"gsm set imei cardnum=%d value=%s\n",gsm_imei.cardnum,gsm_imei.value);
   ctl = open("/dev/dahdi/ctl", O_RDWR);
   if(ctl < 0)
   {
     ast_cli(fd,"gsm set imei open ctl fail\n");
     return RESULT_FAILURE;
   }

   res = ioctl(ctl, DAHDI_GSM_SET_IMEI, &gsm_imei);

   if(res)
   ast_cli(fd,"gsm set imei fail\n");
   else
   ast_cli(fd,"gsm set imei ok\n");
   
   close(ctl);
   return  0;
}

static int gsm_poweroff(int fd, int argc, char *argv[])
{
   
    int ctl;
    int res = 0;
    int cardnum;

   if(argc != 3)
     return RESULT_SHOWUSAGE;

   cardnum = atoi(argv[2]);
  
   if(cardnum < 0 || cardnum >=4)
      return RESULT_SHOWUSAGE;
   
   
   ast_cli(fd,"gsm moudle poweroff cardnum=%d\n",cardnum);
   ctl = open("/dev/dahdi/ctl", O_RDWR);
   if(ctl < 0)
   {
     ast_cli(fd,"gsm poweroff open ctl fail\n");
     return RESULT_FAILURE;
   }

   res = ioctl(ctl, DAHDI_GSM_POWEROFF, &cardnum);

   if(res)
   ast_cli(fd,"gsm poweroff fail\n");
   else
   ast_cli(fd,"gsm poweroff ok\n");
   
   close(ctl);
   return  0;
}

static int gsm_poweron(int fd, int argc, char *argv[])
{
    int ctl;
    int res = 0;
    int cardnum;

   if(argc != 3)
     return RESULT_SHOWUSAGE;

   cardnum = atoi(argv[2]);
  
   if(cardnum < 0 || cardnum >=4)
      return RESULT_SHOWUSAGE;
   
   
   ast_cli(fd,"gsm moudle poweron cardnum=%d\n",cardnum);
   ctl = open("/dev/dahdi/ctl", O_RDWR);
   if(ctl < 0)
   {
     ast_cli(fd,"gsm poweron open ctl fail\n");
     return RESULT_FAILURE;
   }

   res = ioctl(ctl, DAHDI_GSM_POWERON, &cardnum);

   if(res)
   ast_cli(fd,"gsm poweron fail\n");
   else
   ast_cli(fd,"gsm poweron ok\n");

   close(ctl);
   return 0;
}
#endif
#if 0
static struct ast_cli_entry cli_gsm_set_imei_deprecated = {
	{ "gsm", "set", "imei", NULL,NULL },
	gsm_set_imei, NULL,NULL};

static struct ast_cli_entry cli_gsm_poweroff_deprecated = {
	{ "gsm", "poweroff", NULL },
	gsm_poweroff, NULL,NULL};

static struct ast_cli_entry cli_gsm_poweron_deprecated = {
	{ "gsm", "poweron", NULL },
	gsm_poweron, NULL,NULL};


static struct ast_cli_entry cli_gsm[] = {
        { { "gsm", "set", "imei",NULL },
        gsm_set_imei, NULL, NULL,NULL },
    
        { { "gsm", "poweroff", NULL },
        gsm_poweroff, NULL,NULL,NULL },
    
        { { "gsm", "poweron", NULL },
        gsm_poweron, NULL,NULL,NULL},
};
#endif

static int sendsms_exec(struct ast_channel *chan, char *data)
{
    char *chantech;
    char *chandata;
    int timeout = 3000;
    char *app = "DoSms";
    char *parse; 
    char appdata[1400]={0};
  
    AST_DECLARE_APP_ARGS(args,
			     AST_APP_ARG(channum);
			     AST_APP_ARG(phonenum);
			     AST_APP_ARG(content);
	);

      if (ast_strlen_zero(data))
                return -1;

      parse = ast_strdupa(data);
	
      AST_STANDARD_APP_ARGS(args, parse);

      
      if (!ast_strlen_zero(args.channum))
      {
	chandata = ast_strdupa(args.channum);
        chantech = strsep(&chandata, "/");
      
        if (!chandata) {
		ast_log(LOG_WARNING,"sendsms app must set gsm channel num\n"); 
		return -1;
        }
      }
      else
      {
	
	ast_log(LOG_WARNING,"sendsms app must set channum phonenum and content\n"); 
	return -1;
      }

      if (!ast_strlen_zero(args.phonenum))
      {
	 if(strlen(args.content) > 1024)
	 {
	     ast_log(LOG_WARNING,"the length of the sms message is too long is should smaller than 1024 byte\n");
	     return -1;	 
	} 
	else
	 snprintf(appdata,sizeof(appdata),"%s %s",args.phonenum,args.content);
      }	
      else
      {
	ast_log(LOG_WARNING,"sendsms app must set phone num\n"); 
        return -1;
      }
       ast_pbx_runing_app(chantech, AST_FORMAT_SLINEAR, chandata, timeout, app, appdata);
       return 0;
}

static int sms_exec(struct ast_channel *chan, char *data)
{
    char *status = "FAILURE";

	if (!data) {
		ast_log(LOG_WARNING, "Sendsms requires an argument (text)\n");
		return -1;
	} 

   if (ast_sendtext(chan, data)) {
        status = "SUCCESS";
    }

	pbx_builtin_setvar_helper(chan, "SENDSMSSTATUS", status);

	return 0;
}

static int unload_module(void)
{
	int retval;
	retval =  ast_unregister_application(app);
	retval |=  ast_unregister_application(app1);
	return retval;
}

static int load_module(void)
{
	//return ast_register_application_xml(app, sendsms_exec);
	int retval;
//        ast_cli_register_multiple(cli_gsm, sizeof(cli_gsm) / sizeof(struct ast_cli_entry));
        retval = ast_register_application(app, sendsms_exec, sendsms_synopsis, sendsms_descrip);
        retval |= ast_register_application(app1, sms_exec, sms_synopsis, sms_descrip);
	return retval;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Send sms Applications");
