/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2005 - 2006, Digium, Inc.
 *
 * Russell Bryant <russell@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*!
 * \file
 * \author atcom
 *
 * \brief Send sms via the CLI
 *
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 196758 $");

#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/cli.h"
#include "asterisk/utils.h"
#include "asterisk/frame.h"

    
/*!
 * \brief send a sms from the CLI
 * \param fd file descriptor for cli
 * \param chan channel to create type/data
 * \param app application you want to run
 * \param appdata data for application
 * \retval CLI_SUCCESS on success.
 * \retval CLI_SHOWUSAGE on failure.
*/

static char send_msg_usage[] = 
"	 A sms can be send by a channel, This is similar to sendtext\n"
			"	Usage: sendsms caller(tech/peername)  <dahdi/id>  phone  message\n";



static int sendsms(int fd, const char *chan, const char *appdata,const char *req)
{
	char *chantech;
	char *chandata;
    int timeout = 3000;
    char *app = "Sendsms";
    char *ast_app =  pbx_findapp(app);
     if (!ast_app)
		return -1;

	if (ast_strlen_zero(appdata))
		return -1;

	chandata = ast_strdupa(req);
        
         
	chantech = strchr(chandata,'/');
          
	if (!chantech) {
		ast_cli(fd, "*** No data provided after channel type! ***\n");
		return -1;
	}
        *chantech++='\0';
        
        ast_pbx_runing_app("sip", AST_FORMAT_SLINEAR,chandata, timeout, app, appdata); 
	return 0;
}


/*!
 * \brief handle for send out.
 * \param e pointer to the CLI structure to initialize
 * \param cmd operation to execute
 * \param a structure that contains either application or extension arguments
 * \retval CLI_SUCCESS on success.
 * \retval CLI_SHOWUSAGE on failure.*/
static int handle_sendsms(int fd, int argc, char *argv[])
{
    int res ;
    int i;
    char tmp[1024]={0};
    char data[1024]={0};

     if(argc < 5)
     {
       return RESULT_SHOWUSAGE;
     }

     if (ast_strlen_zero(argv[1]) || ast_strlen_zero(argv[2])) 
     {
		return RESULT_SHOWUSAGE;
     }
	/* ugly, can be removed when CLI entries have ast_module pointers */
    ast_module_ref(ast_module_info->self);

    strncpy(data, argv[3], sizeof(data));

    for (i=4; i<argc; i++){
        if (strlen(data) >= sizeof(data)) 
            break;
        ast_copy_string(tmp, data,sizeof(tmp));
        if(i>4)
        snprintf(data, sizeof(data), "%s %s", tmp, argv[i]);
        else
        snprintf(data, sizeof(data), "%s|%s", tmp, argv[i]);
    }
    memset(tmp,0,sizeof(tmp));

    snprintf(tmp, sizeof(tmp), "%s|%s",argv[2], data);
    if (!strncasecmp("dahdi", argv[2], 5)) 
		res = sendsms(fd, argv[2], tmp,argv[1]);
	else 
		res = -1;

	ast_module_unref(ast_module_info->self);

	return res;
}
static struct ast_cli_entry cli_sendsms[] = {
	{ { "sendsms", NULL, NULL, NULL,NULL, },
	handle_sendsms, "Send a sms",
	send_msg_usage, NULL, },	
};

#if 0
static struct ast_cli_entry cli_sendsms[] = {
	AST_CLI_DEFINE(handle_sendsms, "Send a sms"),
};
#endif

static int unload_module(void)
{
	ast_cli_unregister_multiple(cli_sendsms, ARRAY_LEN(cli_sendsms));
	return 0;
}

static int load_module(void)
{
	int res;
	ast_cli_register_multiple(cli_sendsms, ARRAY_LEN(cli_sendsms));
	return 0;
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Send a sms via the CLI");
